@@ -1,15 +1,16 @@
 import re
+import warnings
 from django.conf import settings
-from threading import local
 from django.utils.importlib import import_module
 from django.core.exceptions import ImproperlyConfigured
-from django.db import utils
 from tenant_schemas.utils import get_public_schema_name
 
 ORIGINAL_BACKEND = getattr(settings, 'ORIGINAL_BACKEND', 'django.db.backends.postgresql_psycopg2')
 
 original_backend = import_module('.base', ORIGINAL_BACKEND)
 
+EXTRA_SEARCH_PATHS = getattr(settings, 'PG_EXTRA_SEARCH_PATHS', [])
+
 # from the postgresql doc
 SQL_IDENTIFIER_RE = re.compile('^[_a-zA-Z][_a-zA-Z0-9]{,62}$')
 
@@ -19,60 +20,16 @@ def _check_identifier(identifier):
         raise RuntimeError("Invalid string used for the schema name.")
 
 
-class PGThread(local):
+class DatabaseWrapper(original_backend.DatabaseWrapper):
     """
-    Indicates if the public schema should be included on the search path.
-    When syncing the db for creating the tables, it's useful to exclude
-    the public schema so that all tables will be created.
-
-    This is a bad monkey patching resulting from the fact that we can't
-    separate public from shared apps right now. issue #1 on github
+    Adds the capability to manipulate the search_path using set_tenant and set_schema_name
     """
     include_public_schema = True
 
-    def __init__(self):
+    def __init__(self, *args, **kwargs):
+        super(DatabaseWrapper, self).__init__(*args, **kwargs)
         self.set_schema_to_public()
 
-    def set_search_path(self, cursor):
-        """
-        Actual search_path modification for the cursor. Database will
-        search schemata from left to right when looking for the object
-        (table, index, sequence, etc.).
-        """
-
-        if self.schema_name is None:
-            raise ImproperlyConfigured("Database schema not set. Did your forget "
-                                       "to call set_schema() or set_tenant()?")
-
-        _check_identifier(self.schema_name)
-        try:
-            public_schema_name = get_public_schema_name()
-            if self.schema_name == public_schema_name:
-                cursor.execute('SET search_path = %s' % public_schema_name)
-            elif self.include_public_schema:
-                cursor.execute('SET search_path = %s,%s', [self.schema_name, public_schema_name])
-            else:
-                cursor.execute('SET search_path = %s', [self.schema_name])
-        except utils.DatabaseError, e:
-            raise utils.DatabaseError(e.message)
-
-        return cursor
-
-    def get_schema(self):
-        return self.schema_name
-
-    def get_tenant(self):
-        return self.tenant
-
-    def set_schema(self, schema_name, include_public=True):
-        """
-        Main API method to current database schema,
-        but it does not actually modify the db connection.
-        """
-        self.tenant = None
-        self.schema_name = schema_name
-        self.include_public_schema = include_public
-
     def set_tenant(self, tenant, include_public=True):
         """
         Main API method to current database schema,
@@ -81,46 +38,38 @@ class PGThread(local):
         self.tenant = tenant
         self.schema_name = tenant.schema_name
         self.include_public_schema = include_public
+        self.set_settings_schema(self.schema_name)
 
-        if self.tenant is not None:
-            if self.schema_name != self.tenant.schema_name:
-                raise ImproperlyConfigured("Passed schema '%s' does not match tenant's schema '%s'."
-                                           % (self.schema_name, self.tenant.schema_name))
+    def set_schema(self, schema_name, include_public=True):
+        """
+        Main API method to current database schema,
+        but it does not actually modify the db connection.
+        """
+        self.tenant = FakeTenant(schema_name=schema_name)
+        self.schema_name = schema_name
+        self.include_public_schema = include_public
+        self.set_settings_schema(schema_name)
 
     def set_schema_to_public(self):
         """
         Instructs to stay in the common 'public' schema.
         """
-        self.tenant = None
+        self.tenant = FakeTenant(schema_name=get_public_schema_name())
         self.schema_name = get_public_schema_name()
-
-
-class DatabaseWrapper(original_backend.DatabaseWrapper):
-    def __init__(self, *args, **kwargs):
-        super(DatabaseWrapper, self).__init__(*args, **kwargs)
-
-        self.pg_thread = PGThread()
-
-    def set_tenant(self, tenant, include_public = True):
-        self.set_settings_schema(tenant.schema_name)
-        self.pg_thread.set_tenant(tenant, include_public)
-
-    def set_schema(self, schema_name, include_public = True):
-        self.set_settings_schema(schema_name)
-        self.pg_thread.set_schema(schema_name, include_public)
-
-    def set_schema_to_public(self):
-        self.set_settings_schema(get_public_schema_name())
-        self.pg_thread.set_schema_to_public()
-
+        self.set_settings_schema(self.schema_name)
+        
     def set_settings_schema(self, schema_name):
         self.settings_dict['SCHEMA'] = schema_name
 
     def get_schema(self):
-        return self.pg_thread.get_schema()
+        warnings.warn("connection.get_schema() is deprecated, use connection.schema_name instead.",
+                      category=DeprecationWarning)
+        return self.schema_name
 
     def get_tenant(self):
-        return self.pg_thread.get_tenant()
+        warnings.warn("connection.get_tenant() is deprecated, use connection.tenant instead.",
+                      category=DeprecationWarning)
+        return self.tenant
 
     def _cursor(self):
         """
@@ -128,8 +77,36 @@ class DatabaseWrapper(original_backend.DatabaseWrapper):
         must go through this to get the cursor handle. We change the path.
         """
         cursor = super(DatabaseWrapper, self)._cursor()
-        cursor = self.pg_thread.set_search_path(cursor)
+
+        # Actual search_path modification for the cursor. Database will
+        # search schemata from left to right when looking for the object
+        # (table, index, sequence, etc.).
+        if not self.schema_name:
+            raise ImproperlyConfigured("Database schema not set. Did you forget "
+                                       "to call set_schema() or set_tenant()?")
+        _check_identifier(self.schema_name)
+        public_schema_name = get_public_schema_name()
+        search_paths = []
+
+        if self.schema_name == public_schema_name:
+            search_paths = [public_schema_name]
+        elif self.include_public_schema:
+            search_paths = [self.schema_name, public_schema_name]
+        else:
+            search_paths = [self.schema_name]
+
+        search_paths.extend(EXTRA_SEARCH_PATHS)
+        cursor.execute('SET search_path = {}'.format(','.join(search_paths)))
         return cursor
 
+
+class FakeTenant:
+    """
+    We can't import any db model in a backend (apparently?), so this class is used
+    for wrapping schema names in a tenant-like structure.
+    """
+    def __init__(self, schema_name):
+        self.schema_name = schema_name
+
 DatabaseError = original_backend.DatabaseError
 IntegrityError = original_backend.IntegrityError
