@@ -7,7 +7,7 @@ from django.core import mail
 
 @contextmanager
 def schema_context(schema_name):
-    previous_tenant = connection.get_tenant()
+    previous_tenant = connection.tenant
     try:
         connection.set_schema(schema_name)
         yield
@@ -17,7 +17,7 @@ def schema_context(schema_name):
 
 @contextmanager
 def tenant_context(tenant):
-    previous_tenant = connection.get_tenant()
+    previous_tenant = connection.tenant
     try:
         connection.set_tenant(tenant)
         yield
@@ -37,8 +37,10 @@ def clean_tenant_url(url_string):
     """
     Removes the TENANT_TOKEN from a particular string
     """
-    if settings.PUBLIC_SCHEMA_URL_TOKEN and url_string.startswith(settings.PUBLIC_SCHEMA_URL_TOKEN):
-        url_string = url_string[len(settings.PUBLIC_SCHEMA_URL_TOKEN):]
+    if hasattr(settings, 'PUBLIC_SCHEMA_URLCONF'):
+        if (settings.PUBLIC_SCHEMA_URLCONF
+                and url_string.startswith(settings.PUBLIC_SCHEMA_URLCONF)):
+            url_string = url_string[len(settings.PUBLIC_SCHEMA_URLCONF):]
     return url_string
 
 
